@@ -163,6 +163,26 @@ class MidiTrack(list):
         return '<midi track {!r} {} messages>'.format(self.name, len(self))
 
 
+def _merge_tracks(tracks):
+    """Merge all messates from tracks.
+
+    Delta times are converted to absolute time (in ticks), and
+    messages from all tracks are sorted on absolute time.
+    """
+    messages = []
+    for i, track in enumerate(tracks):
+        now = 0
+        for message in track:
+            now += message.time
+            messages.append(message.copy(time=now))
+            if message.type == 'end_of_track':
+                break
+
+    messages.sort(key=lambda x: x.time)
+
+    return messages
+
+
 class MidiFile:
     def __init__(self, filename=None, type=1,
                  ticks_per_beat=DEFAULT_TICKS_PER_BEAT,
@@ -309,57 +329,20 @@ class MidiFile:
 
         return track
 
-    def _merge_tracks(self, tracks):
-        """Merge all messates from tracks.
-
-        Delta times are converted to absolute time (in ticks), and
-        messages from all tracks are sorted on absolute time."""
-        messages = []
-        for i, track in enumerate(self.tracks):
-            now = 0
-            for message in track:
-                if message.type == 'end_of_track':
-                    break
-                now += message.time
-                messages.append(message.copy(time=now))
-
-        messages.sort(key=lambda x: x.time)
-
-        return messages
-
     @property
     def length(self):
-        """
-        Playback time in seconds.
+        """Playback time in seconds.
 
         This will be computed by going through every message in every
         track and adding up delta times.
         """
-        # Todo: should fail if type == 2.
-        #       (There's no way to know where each track starts.)
-
         if self.type == 2:
             raise ValueError('impossible to compute length'
                              ' for type 2 (asynchronous) file')
 
-        if not self.tracks:
-            return 0.0
+        return sum(message.time for message in self)
 
-        track_lengths = []
-    
-        for track in self.tracks:
-            seconds_per_tick = self._compute_seconds_per_tick(500000)
-            length = 0.0
-            for message in track:
-                length += (message.time * seconds_per_tick)
-                if message.type == 'set_tempo':
-                    seconds_per_tick = self._compute_seconds_per_tick(
-                        message.tempo)
-            track_lengths.append(length)
-
-        return max(track_lengths)
-
-    def _compute_seconds_per_tick(self, tempo):
+    def _get_seconds_per_tick(self, tempo):
         # Tempo is given in microseconds per beat (default 500000).
         # At this tempo there are (500000 / 1000000) == 0.5 seconds
         # per beat. At the default resolution of 480 ticks per beat
@@ -375,21 +358,24 @@ class MidiFile:
         if self.type == 2:
             raise TypeError('type 2 file can not be played back like this')
 
-        seconds_per_tick = self._compute_seconds_per_tick(DEFAULT_TEMPO)
-        time_of_last_message = 0.0  # seconds
+        seconds_per_tick = self._get_seconds_per_tick(DEFAULT_TEMPO)
+        last_message_time = 0  # ticks
 
-        for message in self._merge_tracks(self.tracks):
+        for message in _merge_tracks(self.tracks):
             # Convert message time from absolute time
             # in ticks to relative time in seconds.
-            now = message.time * seconds_per_tick
-            message.time = max(0.0, now - time_of_last_message)
+            delta = message.time - last_message_time
+            if delta > 0:
+                message.time = delta * seconds_per_tick
+            else:
+                message.time = 0
 
             yield message       
 
-            time_of_last_message = now
+            last_message_time += delta
 
             if message.type == 'set_tempo':
-                seconds_per_tick = self._compute_seconds_per_tick(message.tempo)
+                seconds_per_tick = self._get_seconds_per_tick(message.tempo)
 
     def play(self, meta_messages=False):
         """Play back all tracks.
